#include <algorithm>
#include <cstdio>
#include <random>
#include <vector>
using namespace std;

#ifndef DG
#define DG 0
#endif
#define LOG(...) (DG ? fprintf(stderr, __VA_ARGS__) : 0)

const bool logQuery = false;
const bool logRoots = false;
const bool logDump = false;
const bool logCycle = false;

#define MOD 1000000009

struct Node {
  int name;
  bool ends;
  int priority;
  int size;
  Node* left;
  Node* right;
  Node* parent;
};

int getsize(Node* v) { return v ? v->size : 0; }

Node* merge(Node* a, Node* b) {
  if (!a) return b;
  if (!b) return a;

  if (a->priority > b->priority) {
    a->size += b->size;
    a->right = merge(a->right, b);
    a->right->parent = a;
    return a;
  } else {
    b->size += a->size;
    b->left = merge(a, b->left);
    b->left->parent = b;
    return b;
  }
}

pair<Node*, Node*> split(Node* v, int i) {
  if (!v) return make_pair(nullptr, nullptr);
  int roffset = getsize(v->left) + 1;
  Node* rest;
  if (i >= roffset) {
    tie(v->right, rest) = split(v->right, i - roffset);
    if (v->right) v->right->parent = v;
    if (rest) rest->parent = nullptr;
    v->size = getsize(v->left) + getsize(v->right) + 1;
    return make_pair(v, rest);
  } else {
    tie(rest, v->left) = split(v->left, i);
    if (rest) rest->parent = nullptr;
    if (v->left) v->left->parent = v;
    v->size = getsize(v->left) + getsize(v->right) + 1;
    return make_pair(rest, v);
  }
}

int getindex(Node* v) {
  int i = getsize(v->left);
  while (v->parent) {
    if (v == v->parent->right) i += getsize(v->parent->left) + 1;
    v = v->parent;
  }
  return i;
}

vector<int> phantom_edges;
vector<Node> nodes;
int roots;

Node* findroot(int a) {
  Node* n = &nodes[a * 2];
  while (n->parent) n = n->parent;
  return n;
}

int nameoftree(Node* n) {
  while (n->left) n = n->left;
  return n->name;
}

bool simplequery(int a, int b) {
  if (a == b) return true;
  if (findroot(a) != findroot(b)) return false;
  int abegin = getindex(&nodes[a * 2]);
  int bbegin = getindex(&nodes[b * 2]);
  int bend = getindex(&nodes[b * 2 + 1]);
  if (logQuery) LOG("(%d,%d,%d)", bbegin, abegin, bend);
  return bbegin <= abegin && abegin <= bend;
}

bool query(int a, int b) {
  if (a != b && simplequery(a, b)) return true;
  if (logQuery) LOG(".");
  int c = phantom_edges[nameoftree(findroot(a))];
  if (c != -1 && simplequery(c, b)) return true;
  return false;
}

void edgeadd(int a, int b) {
  if (simplequery(b, a)) {
    if (logCycle) LOG("CYCLE! %d %d\n", a+1, b+1);
    phantom_edges[a] = b;
    return;
  }

  roots--;
  if (logRoots && roots < 1000) LOG("r%d ", roots);
  Node* atree = findroot(a);
  Node* btree = findroot(b);
  int bend = getindex(&nodes[b * 2 + 1]);
  Node *bbefore, *bafter;
  tie(bbefore, bafter) = split(btree, bend);
  merge(merge(bbefore, atree), bafter);
}

void edgeremove(int a) {
  if (phantom_edges[a] != -1) {
    phantom_edges[a] = -1;
    return;
  }

  Node* tree = findroot(a);
  int r = nameoftree(tree);
  if (r == a) return;
  roots++;
  if (logRoots && roots < 1000) LOG("r%d ", roots);
  int c = phantom_edges[r];
  bool incycle = c != -1 && simplequery(c, a);

  int abegin = getindex(&nodes[a * 2]);
  int aend = getindex(&nodes[a * 2 + 1]);

  Node *before, *after;
  tie(tree, after) = split(tree, aend + 1);
  tie(before, tree) = split(tree, abegin);
  merge(before, after);

  if (incycle) {
    phantom_edges[r] = -1;
    edgeadd(r, c);
  }
}

void dump(Node* v) {
  if (!v) {
    LOG("-");
    return;
  }

  LOG("(");
  dump(v->left);
  LOG(" %s%d ", v->ends ? "/" : "", v->name + 1);
  dump(v->right);
  LOG(")");
}

void dumpall() {
  for (Node& node : nodes) {
    if (!node.parent) {
      dump(&node);
      int p = phantom_edges[nameoftree(&node)];
      if (p != -1) LOG(" -> %d", p + 1);
      LOG("\n");
    }
  }
}

int main() {
  int N, Q;
  scanf("%d%d", &N, &Q);

  vector<int> priorities(2 * N);
  for (int i = 0; i < 2 * N; i++) priorities[i] = i;
  default_random_engine rnd(4747);
  shuffle(priorities.begin(), priorities.end(), rnd);

  phantom_edges.resize(N, -1);
  nodes.resize(2 * N);
  roots = N;

  for (int i = 0; i < 2 * N; i++) {
    nodes[i].name = i / 2;
    nodes[i].ends = i % 2;
    nodes[i].priority = priorities[i];
    nodes[i].size = 1;
    nodes[i].left = nodes[i].right = nodes[i].parent = nullptr;
    if (i % 2) merge(&nodes[i - 1], &nodes[i]);
  }

  long long R = 0;

  while (Q--) {
    int t, a, b;
    scanf("%d%d%d", &t, &a, &b);
    a--;
    b--;
    if (t == 1) {
      edgeremove(a);
      if (b != -1) edgeadd(a, b);
    } else {
      if (DG && logDump) dumpall();
      if (logQuery) LOG("[");
      bool answer = query(a, b);
      if (logQuery) LOG("%d]\n", answer);
      R *= 2;
      if (answer) R++;
      R %= MOD;
    }
  }

  printf("%lld\n", R);
}
